/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package arcgissamples.soe.restclient;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;

import sun.misc.BASE64Encoder;

public class PageLayoutRESTClient
{
	private String serverName = "";
	private String mapServiceName = "";
	private String soeName = "";
	private String userName = "";
	private String password = "";
	
	public PageLayoutRESTClient(String serverName, String mapServiceName, String soeName, String userName, String password)
	{
		this.serverName = serverName;
		this.mapServiceName = mapServiceName;
		this.soeName = soeName;
		this.userName = userName;
		this.password = password;
	}
	
	public static void main(String[] args)
	{
		String serverName = "";
		String mapServiceName = "";
		String soeName = "";
		String userName = "";
		String password = "";
		
		if(args.length == 5)
		{
			serverName = args[0];
			mapServiceName = args[1];
			soeName = args[2];
			userName = args[3];
			password = args[4];
		}
		else
		{
			System.out.println("Wrong Usage. For correct usage, see following: \n"
					+ "\nUsage: PageLayoutRESTClient [server name] [map service name] [soe name] [user name] [password]\n\n"
					+ "[server name] - Specifies name of ArcGIS Server\n"
					+ "[map service name] - Specifies name of map service\n"
					+ "[soe name] - Specifies SOE name\n"
					+ "[username] - Specifies user name\n"
					+ "[password] - Specifies user password\n");
			System.exit(1);
		}
		
		try
		{
			PageLayoutRESTClient client = new PageLayoutRESTClient(serverName, mapServiceName, soeName, userName, password);
			client.consumeRESTSOE();
		}
		catch (MalformedURLException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	public void consumeRESTSOE() throws MalformedURLException, IOException
	{
		//get SOE's JSON representation 
		String url = "http://" + this.serverName + ":8399/arcgis/rest/services/" + this.mapServiceName + "/MapServer/exts/" + this.soeName + "?f=json";
		System.out.println("\nSOE's JSON representation: \n" + sendReceive(url));
		
		//invoke sub resource
		url = "http://" + this.serverName + ":8399/arcgis/rest/services/" + this.mapServiceName + "/MapServer/exts/" + this.soeName + "/PageLayout1024x768?f=json";
		System.out.println("\nSub Resource: \n" + sendReceive(url));
		
		//invoke operation
		url = "http://" + this.serverName + ":8399/arcgis/rest/services/" + this.mapServiceName + "/MapServer/exts/" + this.soeName + "/getPageLayoutBySize?" +
				"width=1024&height=800&dpi=72&f=json";
		System.out.println("\nOperation: \n" + sendReceive(url));
	}
	
	private String sendReceive(String urlString) throws MalformedURLException, IOException
	{
		//create HTTP connection
		URL url = new URL(urlString);
		HttpURLConnection httpConnection = getHTTPConnection(url, "GET", this.userName, this.password);
		System.out.println("Header Info: \n");
		printHTTPHeaderInfo(httpConnection);
        
        //send request
        httpConnection.connect();
        
        //get response
        String response = getResponse(httpConnection);
        
        //disconnect
        httpConnection.disconnect();        
		
		return response;
	}	
	
	/**
	 * Prints HTTP Header info
	 * @param httpConnection
	 */
	private void printHTTPHeaderInfo(HttpURLConnection httpConnection)
	{
        // look at headers
        // the 0th header has a null key, and the value is the response line ("HTTP/1.1 200 OK" or whatever)
        String header = null;
        String headerValue = null;
        int index = 0;
        while ((headerValue = httpConnection.getHeaderField(index)) != null)
        {
            header = httpConnection.getHeaderFieldKey(index);
            
            if (header == null)
                System.out.println(headerValue);
            else
                System.out.println(header + ": " + headerValue);
            
            index++;
        }
	}	
	
	/**
	 * Returns an HTTPURLConnection object based on specified URL and HTTP method, username and password
	 * @param url
	 * @param method
	 * @return
	 */
	private HttpURLConnection getHTTPConnection(URL url, String httpMethod, String userName, String userPassword)
	{
		try
		{
			//boolean quiet, String httpMethod, URL url, String username, String password, InputStream body)
			HttpURLConnection httpConnection = (HttpURLConnection) url.openConnection();
			httpConnection.setRequestMethod(httpMethod);
			
			// write auth header
			BASE64Encoder encoder = new BASE64Encoder();
			String encodedCredential = encoder.encode( (userName + ":" + userPassword).getBytes() );
			httpConnection.setRequestProperty("Authorization", "BASIC " + encodedCredential);
			
			return httpConnection;
		}
		catch (ProtocolException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * Returns response from an HTTPConnection object
	 * @param httpConnection
	 * @return
	 */
	private String getResponse(HttpURLConnection httpConnection)
	{
        try
		{
			InputStream responseBodyStream = httpConnection.getInputStream();
			StringBuffer responseBody = new StringBuffer();
			int read = 0;
			byte buffer[] = new byte[8192];
			while ((read = responseBodyStream.read(buffer)) != -1)
			{
			    responseBody.append(new String(buffer, 0, read));
			}
			
			responseBody.trimToSize();
			return responseBody.toString();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		
		return null;
	}		

}
